=== modified file 'loggerhead/controllers/filediff_ui.py'
--- loggerhead/controllers/filediff_ui.py	2009-03-17 06:43:34 +0000
+++ loggerhead/controllers/filediff_ui.py	2009-03-19 10:03:21 +0000
@@ -3,6 +3,7 @@
 
 from bzrlib import diff
 from bzrlib import errors
+from bzrlib import osutils
 
 from loggerhead import util
 from loggerhead.controllers import TemplatedBranchView
@@ -52,25 +53,23 @@
     return chunks
 
 
-def diff_chunks_for_file(file_id, old_tree, new_tree):
-    try:
-        old_lines = old_tree.get_file_lines(file_id)
-    except errors.NoSuchId:
-        old_lines = []
-    try:
-        new_lines = new_tree.get_file_lines(file_id)
-    except errors.NoSuchId:
-        new_lines = []
+def diff_chunks_for_file(repository, file_id, compare_revid, revid):
+    lines = {}
+    args = []
+    for r in (compare_revid, revid):
+        if r == 'null:':
+            lines[r] = []
+        else:
+            args.append((file_id, r, r))
+    for r, bytes_iter in repository.iter_files_bytes(args):
+        lines[r] = osutils.split_lines(bytes_iter)
     buffer = StringIO()
-    if old_lines != new_lines:
-        try:
-            diff.internal_diff('', old_lines, '', new_lines, buffer)
-        except errors.BinaryFile:
-            difftext = ''
-        else:
-            difftext = buffer.getvalue()
+    try:
+        diff.internal_diff('', lines[compare_revid], '', lines[revid], buffer)
+    except errors.BinaryFile:
+        difftext = ''
     else:
-        difftext = ''
+        difftext = buffer.getvalue()
 
     return _process_diff(difftext)
 
@@ -87,9 +86,8 @@
         file_id = urllib.unquote(self.args[2])
 
         repository = self._history._branch.repository
-        old_tree, new_tree = repository.revision_trees([compare_revid, revid])
-
-        chunks = diff_chunks_for_file(file_id, old_tree, new_tree)
+        chunks = diff_chunks_for_file(
+            self._history._branch.repository, file_id, compare_revid, revid)
 
         return {
             'util': util,

=== modified file 'loggerhead/controllers/revision_ui.py'
--- loggerhead/controllers/revision_ui.py	2009-03-19 00:44:05 +0000
+++ loggerhead/controllers/revision_ui.py	2009-03-19 19:04:40 +0000
@@ -70,32 +70,27 @@
         change = h.get_changes([revid])[0]
 
         if compare_revid is None:
-            if change.parents:
-                cr = change.parents[0].revid
-            else:
-                cr = 'null:'
+            file_changes = h.get_file_changes(change)
         else:
-            cr = compare_revid
+            file_changes = h.file_changes_for_revision_ids(
+                compare_revid, change.revid)
 
         if path in ('', '/'):
             path = None
 
-        old_tree = h._branch.repository.revision_tree(cr)
-        new_tree = h._branch.repository.revision_tree(change.revid)
-        file_changes = h.file_changes_from_revision_trees(
-            old_tree, new_tree)
-
         link_data = {}
         path_to_id = {}
         if path:
+            item = [x for x in file_changes.text_changes if x.filename == path][0]
             diff_chunks = diff_chunks_for_file(
-                new_tree.path2id(path), old_tree, new_tree)
+                self._history._branch.repository, item.file_id,
+                item.old_revision, item.new_revision)
         else:
             diff_chunks = None
             for i, item in enumerate(file_changes.text_changes):
                 item.index = i
                 link_data['diff-' + str(i)] = '%s/%s/%s' % (
-                    dq(revid), dq(cr), dq(item.file_id))
+                    dq(item.new_revision), dq(item.old_revision), dq(item.file_id))
                 path_to_id[item.filename] = 'diff-' + str(i)
 
         h.add_branch_nicks(change)

=== modified file 'loggerhead/history.py'
--- loggerhead/history.py	2009-03-19 03:12:58 +0000
+++ loggerhead/history.py	2009-03-19 10:11:51 +0000
@@ -131,18 +131,29 @@
         return len(self.revid_list)
 
 class FileChangeReporter(object):
-    def __init__(self):
+    def __init__(self, old_inv, new_inv):
         self.added = []
         self.modified = []
         self.renamed = []
         self.removed = []
         self.text_changes = []
+        self.old_inv = old_inv
+        self.new_inv = new_inv
+
+    def revid(self, inv, file_id):
+        try:
+            return inv[file_id].revision
+        except bzrlib.errors.NoSuchId:
+            return 'null:'
+
     def report(self, file_id, paths, versioned, renamed, modified,
                exe_change, kind):
         if modified not in ('unchanged', 'kind changed'):
             self.text_changes.append(util.Container(
                 filename=rich_filename(paths[1], kind[0]),
-                file_id=file_id))
+                file_id=file_id,
+                old_revision=self.revid(self.old_inv, file_id),
+                new_revision=self.revid(self.new_inv, file_id)))
         if versioned == 'added':
             self.added.append(util.Container(
                 filename=rich_filename(paths[1], kind),
@@ -588,11 +599,10 @@
     def get_file_changes_uncached(self, entry):
         repo = self._branch.repository
         if entry.parents:
-            old_tree = repo.revision_tree(entry.parents[0].revid)
+            old_revid = entry.parents[0].revid
         else:
-            old_tree = repo.revision_tree(bzrlib.revision.NULL_REVISION)
-        new_tree = repo.revision_tree(entry.revid)
-        return self.file_changes_from_revision_trees(old_tree, new_tree)
+            old_revid = bzrlib.revision.NULL_REVISION
+        return self.file_changes_for_revision_ids(old_revid, entry.revid)
 
     def get_file_changes(self, entry):
         if self._file_change_cache is None:
@@ -614,7 +624,7 @@
             path = '/' + path
         return path, inv_entry.name, rev_tree.get_file_text(file_id)
 
-    def file_changes_from_revision_trees(self, old_tree, new_tree):
+    def file_changes_for_revision_ids(self, old_revid, new_revid):
         """
         Return a nested data structure containing the changes in a delta::
 
@@ -627,7 +637,15 @@
             ),
             text_changes: list((filename, file_id)),
         """
-        reporter = FileChangeReporter()
+        repo = self._branch.repository
+        if bzrlib.revision.is_null(old_revid) or \
+               bzrlib.revision.is_null(new_revid):
+            old_tree, new_tree = map(
+                repo.revision_tree, [old_revid, new_revid])
+        else:
+            old_tree, new_tree = repo.revision_trees([old_revid, new_revid])
+
+        reporter = FileChangeReporter(old_tree.inventory, new_tree.inventory)
 
         bzrlib.delta.report_changes(new_tree.iter_changes(old_tree), reporter)
 

